function R2=crome(X,y);
% returns matrix of squared correlations for regression on pairs
% input
% X  -  n x p matrix of x-variables
% y  -  n x 1 vector with the y-variable

% get no of variables
p=size(X,2);

% Find all the pairwise correlations.
% The matlab function corrcoef returns all the pairwise correlations
% for the variables in the columns of the matrix that is its argument.
% By joining X and y together as [X y] we get a (p+1)x(p+1) matrix C
% of correlation coefficients, where the top left p x p matrix has
% the pairwise correlations of the variables in X, the final column
% and the final row have the pairwise correlations of each x with y
% (and the bottom right hand element is 1, like all the diagonal entries) 

C=corrcoef([X y]);
 
% extract the p x p correlation matrix of the columns of X
% at the same time setting the diagonal to zero (rather than 1)

rx=C(1:p,1:p)-eye(p);   % correlation matrix of x-vars, with 0's on diagonal

% extract the vector of correlations of x with y 
rxy=C(1:p,p+1);        % vector of simple correlations of y with each x

% compute R-squared for all two-var eqns
% note: the fiddling around on the diagonal avoids 0/0 when regressing on the
%       same variable twice.  R-sq is the square of the simple correlation in
%       this case

% I'll leave you to follow why this gives the required result!

rxy2=(rxy.*rxy)*ones(1,p);
div=ones(p,p)-rx.*rx+eye(p);
R2=(rxy2+rxy2'-2*rx.*(rxy*rxy'))./div;
whos

